@@ -1,8 +1,68 @@
-# pylint: disable=C0301, R1702, R0912, R0914, R0915
+# pylint: disable=C0301, R1702, R0912, R0914, R0915, W1401
 """Module to parse Calligraphy scripts into token representation"""
 
 from __future__ import annotations
 import re
+import os
+from calligraphy_scripting import transpiler
+
+
+def handle_sourcing(contents: str) -> str:
+    """Handle replacing Calligraphy source statements and recursively transpiling other files
+
+    Args:
+        contents (str): Contents of a Calligraphy script
+
+    Returns:
+        str: Contents of the Calligraphy script with the source statements replaced
+    """
+
+    def compile_sourced(matches):
+        here = os.path.dirname(os.path.abspath(__file__))
+        for match in matches:
+            with open(
+                os.path.join(match[0], f"{match[1]}.{match[2]}"), encoding="utf-8"
+            ) as code_file:
+                code_contents = code_file.read()
+            code_contents, inline_indices = handle_line_breaks(code_contents)
+            lines, langs = determine_language(code_contents)
+            transpiled = transpiler.transpile(lines, langs, inline_indices)
+
+            # Add the header to enable functionality
+            with open(
+                os.path.join(here, "data", "header.py"), encoding="utf-8"
+            ) as header_file:
+                header = header_file.read()
+            header = header.replace('sys.argv = "PROGRAM_ARGS"', "")
+            code = f"{header}\n\n{transpiled}"
+            with open(
+                os.path.join(match[0], f".{match[1]}.py"), "w", encoding="utf-8"
+            ) as output_file:
+                output_file.write(code)
+
+    source_pattern = r"^[ \t]*source[ \t]+(?:([a-zA-Z0-9_/]*)\/)*([a-zA-Z0-9_]*)\.([a-zA-Z0-9]*)[ \t]*$"
+    source_pattern_rename = r"^[ \t]*source[ \t]+(?:([a-zA-Z0-9_/]*)\/)*([a-zA-Z0-9_]*)\.([a-zA-Z0-9]*)[ \t]as[ \t]([a-zA-Z0-9_]*)[ \t]*"
+
+    sourced = re.findall(source_pattern, contents, re.MULTILINE)
+    sourced_rename = re.findall(source_pattern_rename, contents, re.MULTILINE)
+
+    compile_sourced(sourced)
+    compile_sourced(sourced_rename)
+
+    contents = re.sub(
+        source_pattern,
+        '\g<2> = source_import(os.path.join("\g<1>", "\g<2>"+"."+"\g<3>"), "\g<2>")',
+        contents,
+        flags=re.MULTILINE,
+    )
+    contents = re.sub(
+        source_pattern_rename,
+        '\g<4> = source_import(os.path.join("\g<1>", "\g<2>"+"."+"\g<3>"), "\g<4>")',
+        contents,
+        flags=re.MULTILINE,
+    )
+
+    return contents
 
 
 def get_imports(contents: str) -> list[str]:
@@ -176,13 +236,12 @@ def determine_language(
     assignment_pattern = r"^[ \t]*\(?((?:[a-zA-Z0-9_]+\s*,?\s*)+)\)?[ \t]$"
 
     lines = [line for line in code.split("\n") if len(line.strip()) > 0]
-    variables = ["env."]
+    variables = ["env.", "shellopts."]
     langs = []
-    inline_indices = []
     imports = get_imports(code)
     functions = get_functions(code)
 
-    for idx, line in enumerate(lines):
+    for line in lines:
         is_python = False
         stripped = line.lstrip()
         if stripped.startswith("#") or stripped.startswith('"""'):
@@ -213,8 +272,6 @@ def determine_language(
                     variables += [var.strip() for var in match.group(1).split(",")]
                     inline_match = re.search(inline_bash_pattern, line)
                     if inline_match:
-                        span = inline_match.span()
-                        inline_indices.append((idx, span[0], span[1]))
                         langs.append("MIX")
                         continue
                     langs.append("PYTHON")
@@ -222,15 +279,16 @@ def determine_language(
         else:
             inline_match = re.search(inline_bash_pattern, line)
             if inline_match:
-                span = inline_match.span()
-                inline_indices.append((idx, span[0], span[1]))
                 langs.append("MIX")
                 continue
             langs.append("PYTHON")
             continue
+        if line.strip().startswith("source "):
+            langs.append("CALLIGRAPHY")
+            continue
         langs.append("BASH")
 
-    return lines, langs, inline_indices
+    return lines, langs
 
 
 def handle_line_breaks(code: str) -> str:
@@ -244,15 +302,21 @@ def handle_line_breaks(code: str) -> str:
     """
 
     depths = {"PAREN": 0, "BRACE": 0, "BRACKET": 0}
-
     flags = {"SINGLE_QUOTE": False, "DOUBLE_QUOTE": False}
-
+    inline_indices = []
     output = ""
+    line_idx = 0
+    char_idx = 0
+    in_inline = False
+    inline_paren_depth = 0
 
     for idx, token in enumerate(code):
+        look_ahead = None
         look_behind = None
         if idx > 0:
             look_behind = code[idx - 1]
+        if idx < len(code) - 1:
+            look_ahead = code[idx + 1]
 
         if token == "'" and look_behind != "\\" and not flags["DOUBLE_QUOTE"]:
             flags["SINGLE_QUOTE"] = not flags["SINGLE_QUOTE"]
@@ -273,7 +337,10 @@ def handle_line_breaks(code: str) -> str:
             and not (flags["SINGLE_QUOTE"] or flags["DOUBLE_QUOTE"])
         ):
             depths["PAREN"] -= 1
-
+            if in_inline:
+                if depths["PAREN"] == inline_paren_depth:
+                    inline_indices[-1][2] = char_idx + 1
+                    in_inline = False
         if (
             token == "{"
             and look_behind != "\\"
@@ -311,10 +378,21 @@ def handle_line_breaks(code: str) -> str:
                     or flags["DOUBLE_QUOTE"]
                 ):
                     output += "<CALLIGRAPHY_NEWLINE>"
+                    char_idx += 1
                     continue
+                if char_idx != 0:
+                    line_idx += 1
+                char_idx = -1
             else:
                 output += "<CALLIGRAPHY_NEWLINE>"
+                char_idx += 1
                 continue
+        if token in ("$", "?", "!"):
+            if look_ahead == "(":
+                inline_paren_depth = depths["PAREN"]
+                in_inline = True
+                inline_indices.append([line_idx, char_idx, None])
 
         output += token
-    return output
+        char_idx += 1
+    return output, inline_indices
